<html lang="en" class="layout-pf layout-pf-fixed-with-footer">
  <head>
    <meta charset="utf-8">
    <style>
      .highlight { background-color:#FFFF00; }
    </style>
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/patternfly/4.0.0-rc.1/css/patternfly.min.css" integrity="sha384-w93mNn3exOGKokXXtOJ4to54RqQjfJnA3LtX7GLeZweAJ+1fy83CglzsEdB9sRhq" crossorigin="anonymous">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/patternfly/4.0.0-rc.1/css/patternfly-additions.min.css" integrity="sha384-vMVHK5/I4lTibgeicJh67p9z9daCJmoeaRCcPpYbk6ZAcR8UEIQseTr8bC1BN5+m" crossorigin="anonymous">
    <title>Aegis AI</title>
  </head>
  <body style="background-color: whitesmoke">
    <nav class="navbar navbar-default navbar-pf" role="navigation">
      <div style="margin: 3px 0 5px 20px;">
        <a title="Product Security Agent" href="/"
           style="font-size: 18px; text-decoration: none; color: white">
          <b>Aegis AI - Product Security Agent</b>
        </a>
      </div>
    </nav>
    <div class="container-fluid">
      <hr/>
      <p>Analysis features:<br/>
          CVE Features: GET /api/v1/analysis/cve?feature={feature_name}&amp;cve_id={cve_id}<br/>
          CVE Features: POST /api/v1/analysis/cve/{feature_name}<br/>
          Component Features: GET /api/v1/analysis/component?feature={feature_name}&amp;component_name={component_name}<br/>
          KPI Metrics: GET /api/v1/analysis/kpi/cve?feature={feature_name}&amp;order={asc|desc}<br/>
      </p>
      <hr/>
      <p>Docs:
          <a href='https://github.com/RedHatProductSecurity/aegis-ai'>Aegis Github</a> | <a href='/docs'>API docs</a><br/>
      </p>
      <p>Schema:
          <a href='/openapi.json'>openapi json</a> | <a href='/openapi.yml'>openapi yaml</a><br/>
      </p>
      <p>Debug:
          <a href='/console'>console</a>
      </p>
      <hr/>
      <p>
      <i>Disclaimer: Any responses generated by Aegis AI (https://github.com/RedHatProductSecurity/aegis-ai) is generative AI for informational purposes. All findings should be validated by a human expert.</i>
      </p>
    </div>
    <div class="container-fluid footer-pf">
      <div class="text-center" style="font-size:120%; padding-bottom: 5px">
        Copyright © 2025 Red Hat, Inc. All rights reserved.
        |
        Version: {{ version }}
        |
        <a href="https://github.com/RedHatProductSecurity/aegis-ai/issues"
           rel="noopener noreferrer" target="_blank">
          Report Issue/Enhancement
        </a>
      </div>
    </div>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.5.1/jquery.min.js" integrity="sha384-ZvpUoO/+PpLXR1lu4jmpXWu80pZlYUAfxl5NsBMWOEPSjUn/6Z/hRTt8+pR6L4N2" crossorigin="anonymous"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.4.1/js/bootstrap.min.js" integrity="sha384-aJ21OjlMXNL5UyIl/XNwTMqvzeRMZH2w8c5cRVpzpU8Y5bApTppSuUkhZXN0VxHd" crossorigin="anonymous"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/patternfly/3.59.5/js/patternfly.min.js" integrity="sha384-3eAK2Q6H2vfFZqS7Wx3RCTBf4rj/1Dfs6OlP0iDq5wkkZyoMas2r2WZpuKp53aVQ" crossorigin="anonymous"></script>
  </body>
</html>
